require 'couchrest_model'

######## CLOUDANT INDEX HANDLING ########
class CouchRest::Model::Design
  def create_index(name, function)
    indexes = (self['indexes'] ||= {})
    indexes[name.to_s] = function
  end
end

class CouchRest::Model::Designs::DesignMapper
  def index(name, function)
    design_doc.create_index(name, function)
  end
end

######## CLOUDANT SEARCH QUERYING ######
class CouchRest::Database
  def search(design, index, query, options={})
    CouchRest.get CouchRest.paramify_url("#{@root}/_design/#{design}/_search/#{index}", options.merge(:q => query))
  end
end

class CouchRest::Model::SearchResults
  include Enumerable

  attr_accessor :total_rows, :bookmark

  def initialize(total_rows, bookmark, objects)
    @total_rows = total_rows
    @bookmark = bookmark
    @objects = objects
  end

  def each
    @objects.each do |member|
      yield member
    end
  end
end

# 
# Execute a cloudant search query.
# if you are not using the :include_docs option then your indexes must be named to
# match document properties otherwise they cannot be casted into document models.  
#
class CouchRest::Model::Base
  def self.search(index, query, options={})
    if options[:sort]
      options[:sort] = '"' + options[:sort] + '"'
    end
    if options['sort']
      options['sort'] = '"' + options['sort'] + '"'
    end
    ret = self.database.search(self.to_s, index, query, options)
    rows = ret['rows']
     if options[:include_docs] == true || options['include_docs'] == true
      objects = rows.map {|r| obj = self.new(r['doc']) }
    else
      objects = rows.map {|r| obj = self.new(r['fields']); obj.id = r['id']; obj }
    end
    CouchRest::Model::SearchResults.new(ret['total_rows'], ret['bookmark'], objects)
  end
end